xquery version "3.1";
(:~
 : This module provides an API to retrieve textual commentary entries as a sequence for a
 : given page.
 :
 : @author Simon Sendler
 : @author Mathias Göbel
 : @version 1.0
 : @since 3.2.2
 : :)

module namespace stk="https://fontane-nb.dariah.eu/stk-api";

declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace test="http://exist-db.org/xquery/xqsuite";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace rest="http://exquery.org/ns/restxq";
import module namespace ixp="http://fontane-nb.dariah.eu/index-processor" at "index-processor.xqm";

declare variable $stk:dataPath := "/db/sade-projects/textgrid/data/xml/data";
declare variable $stk:dataCollection := collection($stk:dataPath);

(:~
 : wrapper function to provide a versionized REST-API
 : @see stk:entities-per-page
 :)
declare
  %private
  %rest:GET
  %rest:path("fontane/v1/stk/{$uri}/{$surface}")
  %output:method("html5")
function stk:v1-entities-per-page($uri as xs:string, $surface as xs:string) {
    stk:entities-per-page($uri, $surface)
};

(:~
 : lists all textual commentary on a given page
 :
 : @param $uri – a valid TextGrid-Base-URI without prefix
 : @param $surface – a string to match a tei:surface/@n in the document (use “-” as delimiter)
 : @return a html5 list with all entries
 : @see http://localhost:8080/exist/restxq/fontane/stk/22jtk/57r
 : @see https://fontane-nb.dariah.eu/api/stk/22jtk/57r
 :  :)
declare
  %rest:GET
  %rest:path("fontane/stk/{$uri}/{$surface}")
  %output:method("html5")
function stk:entities-per-page($uri as xs:string, $surface as xs:string)
as element(xhtml:div) {

let $docpath := $stk:dataPath || "/" || $uri || ".xml"
let $doc := doc($docpath)
let $surface := tokenize($surface, "-")
let $reference := $doc//tei:surface[@n = $surface]//@xml:id

return (
    element xhtml:div {
        attribute class {"notes"},
        element xhtml:ul {
            for $note in $doc//tei:note[@type="editorial"]
            where $note/@target => replace("#", "") => tokenize(" ") = $reference
            return
                element xhtml:li {
                    attribute class { 'editorialNote' },
                    stk:noteParser($note)
                }
        }
    })
};

(:~
 : pareses a tei:note and the corresponding target to a html representation form
 : Uses recursion.
 : @param $note – a tei:note to initate
 :)
declare function stk:noteParser($note as node()*) {
    for $node in $note return
        typeswitch ( $node )
        case element( tei:note ) return
            let $xmlids := tokenize(replace($node/@target, "#", ""), " ")
            return (
                attribute data-ref { $xmlids },
                element xhtml:span {
                    attribute class { "target" },
                    string-join(
                        for $xmlid in $xmlids
                        return
                            string-join($node/..//*[@xml:id = $xmlid]//text()
                            [not(parent::tei:expan)],
                            ""), " "
                    ) => replace("- |⸗ ", "")
                    || "]"
                },
                element xhtml:ul {
                    element xhtml:li {
                        stk:noteParser($node/node())
                    }
                })

        case element (tei:bibl) return
            stk:noteParser($node/node())

        case element (tei:citedRange) return (
            ", ",
            if ( starts-with($node, "http") )
                then <a title="citedRange" href="{string($node)}"><i class="fa fa-external-link"></i></a>
            else $node/text())

        case element (tei:ptr) return (
            element xhtml:a {
                let $targetId := substring-after($node/@target, ":")
                return
                (attribute href {"literaturvz.html?id=" || $targetId},
                $stk:dataCollection/id($targetId)/tei:choice/tei:abbr/text())
            },
            stk:noteParser($node/node())
            )

        case element (tei:ref) return
            let $TGuri := string($node/ancestor::tei:TEI/tei:teiHeader//tei:idno[@type="TextGrid"])
            let $uri := $TGuri => substring-after(":")
            let $target := replace($node/@target, "#", "")
            let $pattern := "^#.+_.+_.+$"
            return
                if (matches($node/@target, $pattern)) then
                    element xhtml:a {
                        attribute href { "edition.html?id=%2Fxml%2Fdata%2F"
                            || $uri || ".xml&amp;page=" || $node
                            || "&amp;target=" || $target },
                        stk:noteParser($node/node())
                    }
                else
                    <xhtml:span style="color:red; font-weight:bold">error:
                    tei:ref/@target does not match pattern »{$pattern}« </xhtml:span>

        case element (tei:rs) return
            element xhtml:a {
                attribute href { stk:teiref2href($node/string(@ref)) },
                stk:noteParser($node/node())
            }

        case element (tei:seg) return
            element xhtml:span {
                $node/@style,
                stk:noteParser($node/node())
            }

        case text() return $node

        default return
            stk:noteParser($node/node())
};

(:~
 : prepares a URL for a referenced item form the index
 : @param $ref – a @tei:ref value with prefix
 :)
declare function stk:teiref2href($ref as xs:string) as xs:string {
    let $prefix := substring-before($ref, ":")
    let $id := substring-after($ref, ":")
    return
        switch($prefix)
            case "lit" return
                "literaturvz.html?id=" || $id
            default return
                "register.html?e=" || $id
};
